<?php
get_header();

$location_posts = get_posts( [ 'post_type' => 'location', 'numberposts' => -1 ] );
$locations = [];

$overview_video_id = get_field('overview_video_id');
    
foreach($location_posts as $location_post){
    $coordinates = get_field('coordinates',$location_post->ID);
    
    $location = [
        'title' => $location_post->post_title,
        'id' => $location_post->ID,
        'address_line_1' => get_field( 'address_line_1', $location_post->ID ),
        'address_line_2' => get_field( 'address_line_2', $location_post->ID ),
        'city' => get_field( 'city', $location_post->ID ),
        'state' => get_field( 'state', $location_post->ID ),
        'zip' => get_field( 'zip_code', $location_post->ID ),
        'phone' => get_field( 'phone', $location_post->ID ),
        'fax' => get_field( 'fax', $location_post->ID ),
        'referral_fax' => get_field( 'referral_fax', $location_post->ID ),
        'additional_information' => get_field( 'additional_information', $location_post->ID ),
        'directions_link' => 'https://www.google.com/maps/dir/?api=1&destination=' . urlencode($coordinates['address']) . '&destination_place_id=' . ($coordinates['place_id']),
        'coordinates' => $coordinates
    ];
    
    $schema_location = [
        '@context' => 'http://www.schema.org',
        '@type' => 'MedicalClinic',
        'name' => get_bloginfo('name') . ' - '. $location['title'],
        'url' => get_site_url(),
        'logo' => get_field('structured_data_logo', 'option'),
        'image' => get_field('structured_data_logo', 'option'),
        'description' => get_the_excerpt(),
        'address' => [
            "@type" => "PostalAddress",
            "streetAddress" => $location['address_line_1']. ", " . $location['address_line_2'],
            "addressLocality" => $location['city'],
            "addressRegion" => $location['state'],
            "postalCode" => $location['zip'],
            "addressCountry" => "USA"
        ],
        'geo' => [
            '@type' => 'GeoCoordinates',
            'latitude' => $location['coordinates']['lat'],
            'longitude' => $location['coordinates']['lng']
        ],
        'telephone' => $location['phone'],
        'priceRange' => '$'
    ];
    
    $location['schema']=$schema_location;
    
    $locations[] = $location;

}

$schema = [];

if(count($locations) == 1){
    $locations[0]['schema']['@id'] = get_site_url();
    $schema = $locations[0]['schema'];
}

if(count($locations) > 1){
    $schema = [
        '@context' => 'http://schema.org',
        '@type' => 'Organization',
        'name' => get_bloginfo('name'),
        'logo' => get_field('structured_data_logo', 'option'),
        'image' => get_field('structured_data_logo', 'option'),
        '@id' => get_site_url(),
        'url' => get_site_url(),
        'location'  => array_column($locations,'schema') 
    ];
    
    foreach($schema['location'] as &$schema_location){
        $schema_location['parentOrganization'] = [
            'name' => get_bloginfo('name')
        ];
    }
}
?>

<?php if(count($schema) > 0){ ?>
<script type='application/ld+json'>
    <?= json_encode($schema, JSON_UNESCAPED_SLASHES) ?>
    
</script>
<?php } ?>
<section class="post home-main">
    <div class="header-banner">
        <img src="<?= get_the_post_thumbnail_url( get_the_ID(), 'viewport-width' ) ?>" style="object-position: 50% <?php the_field('featured_image_center_v'); ?>%" 
             srcset="<?= get_the_post_thumbnail_url( get_the_ID(), 'viewport-width' ) ?> 1920w,
                     <?= get_the_post_thumbnail_url( get_the_ID(), 'large' ) ?> 1200w,
                     <?= get_the_post_thumbnail_url( get_the_ID(), 'medium' ) ?> 800w"
             width="1920" height="1080" alt="">        
        <div class="aon-container">
            <h1>
                <?= get_the_title() ?>
            </h1>
            <? if($overview_video_id != '') { ?>
                <button class="overview-video-button" data-id="<?= $overview_video_id; ?>">Overview Video</button>
            <? } ?>
        </div>
    </div>
    <div class="content">
        <div class="aon-container">
            <?php 
				/*wp_nav_menu( array(
					'theme_location' => 'quick_links',					
					'menu_class' => 'quick-links',
					'container_class' => 'quick-links-container'
				) );*/
			?>
        </div>
        <div class="aon-container">
            <?php the_content(); ?>
        </div>
    </div>
</section>
<section class="home-services" id="services">
    <h2 class="section-title">Our Services</h2>
    <div class="service-cards slider">
        <button type="button" class="slider-arrow left" aria-label="view services to the left">&lang;</button>
        <button type="button" class="slider-arrow-cover" aria-label="view services to the right"></button>
        <?php
        foreach ( get_posts( [ 'post_type' => 'service', 'numberposts' => -1 ] ) as $service ) {
            $image = get_the_post_thumbnail_url( $service->ID, 'medium' );
            $image_small = get_the_post_thumbnail_url( $service->ID, 'thumbnail' );
            $link = get_permalink( $service->ID );
            $excerpt = get_the_excerpt( $service->ID );
            $call_to_action = get_field( 'call_to_action', $service->ID );
            ?>
        <article class="service-card"> <img src="<?=$image?>" srcset="<?=$image?> 1600w, <?=$image_small?> 800w" width="320" height="180" alt="Image for <?= $service->post_title;?>" />
            <h3 class="title title3">
                <a href="<?=$link?>"><?= $service->post_title;?></a>
            </h3>            
            <p class="content">
                <?=$excerpt; ?>
            </p>
            <a class="cta" href="<?=$link?>"><?=$call_to_action?>&nbsp;&nbsp;&#62;</a></article>
        <?php
        }
        ?>
        <div class="slider-arrow-cover"></div>
        <div class="slider-arrow right">&rang;</div>
    </div>
</section>
<section class="home-team" id="team">
    <h2 class="section-title">Your Team</h2>
    <?php
    $team_members = get_posts( [ 'post_type' => 'physician', 'numberposts' => -1 ] );
    
    if ( get_field('include_practitioners') ){
        $practitioners = get_posts( [ 'post_type' => 'practitioner', 'numberposts' => -1 ] );
        $team_members = array_merge( $team_members, $practitioners);
    }
    
    if( empty( $team_members ) ){
        ?>
        <div style="text-align:center; font-size: 1.5rem; margin-top: 1rem">Coming Soon!</div>
        <?php
    }else{
        $columns = 5;
        $columns_mobile = count( $team_members );
    
        if ( count( $team_members ) > 2 ) {
            $columns_mobile = 2;
        }
        
        while( ( ( count( $team_members ) % $columns ) <= ($columns / 2) ) && ( count( $team_members ) % $columns ) != 0 && $columns > 1 ){
            $columns--;
        }
        
        
        ?>
        <div class="team-members" style="--columns: <?=$columns?>; --columns-mobile: <?=$columns_mobile?>">
            <?php
            foreach ( $team_members as $team_member ) {
                $image = get_the_post_thumbnail_url( $team_member->ID, 'medium' );
                $link = get_permalink( $team_member->ID );
                $excerpt = get_the_excerpt( $team_member->ID );
                $position = get_field( 'position', $team_member->ID );
                ?>
            <article class="team-member"> <a href="<?=$link?>"> <img src="<?=$image?>" width="320" height="320" alt="Headshot of <?= $team_member->post_title;?>" />
                <h3 class="title title3">
                    <?= $team_member->post_title;?>
                </h3>
                <p class="position">
                    <?=$position; ?>
                </p>
                <span class="read-bio">Read bio ></span>
                </a> </article>
            <?php
            }
            ?>
        </div>
        <?php 
    } 
    ?>
</section>
<section class="home-locations" id="locations"> <?php //echo get_field('google_api_key', 'option'); ?>
    <h2 class="section-title">Location<?= count($locations)>1 ? 's' : '' ?></h2>
    <?php
        $columns = 1;
        
        if ( count( $locations ) >= 3 ) {
            $columns = 5;
        }
        
        while( ( ( count( $locations ) % $columns ) <= ($columns / 2) ) && ( count( $locations ) % $columns ) != 0 && $columns > 1 ){
            $columns--;
        }
        
    ?>
    <div class="<?php echo $columns==1 ? 'aon-container location-small' : 'location-many'; ?>">
        <div class="location-cards" style="--columns: <?=$columns?>;">
            <?php foreach ( $locations as $location ) { ?>
                <article class="location-card">
                    <h3 class="title title3">
                        <?= $location['title'];?>
                    </h3>
                    <p class="address">
                        <?php 
                            echo $location['address_line_1']; 
                            if($location['address_line_2']!=''){ 
                                ?>, <span style="white-space: nowrap"><?= $location['address_line_2'] ?></span><?php 
                            } 
                        ?>
                        <br />
                        <?= $location['city'] ?>, <span style="white-space: nowrap"><?= $location['state'] ?> <?= $location['zip'] ?></span>
                    </p>					
					<?php if( !empty($location['additional_information']) ){ ?>
					<p class="additional-information">
						<?= $location['additional_information'] ?>
					</p>
					<?php } ?>
                    <p class="lines">
                        Phone: <a href="tel:<?= $location['phone'] ?>"><?= $location['phone'] ?></a>
                        <?php if( !empty($location['fax']) ){ ?>
                            <br />
                            Fax: <?= $location['fax'] ?>
                        <?php } ?>
                        <?php if( !empty($location['referral_fax']) ){ ?>
                            <br />
                            Referral Fax: <?= $location['referral_fax'] ?>
                        <?php } ?>
                    </p>
                    
                    <a href="<?= $location['directions_link'] ?>" target="_blank" class="directions" aria-label="Drive using Google Maps">Driving Directions > </a>
                </article>
            <?php
            }
            ?>
        </div>
        <div id="homeMap">
            <pre class="homeMapSettings">
                <?php 
                    $markers = [];

                    foreach($locations as $location){
                        $markers[] = [
                            'position' => [
                                'lat' => $location['coordinates']['lat'],
                                'lng' => $location['coordinates']['lng']
                            ],
                            'title' => $location['title']
                        ]; 
                    }
                    
                    echo json_encode([ 'markers' => $markers, 'maxzoom' => intval( get_field('map_zoom') ) ]); 
                ?>
            </pre>
        </div>
        <script defer src="<?= get_template_directory_uri(); ?>/front-page.js?v=<?= get_field('version', 'option'); ?>"></script>
        <script defer src="https://maps.googleapis.com/maps/api/js?key=<?=get_field('google_api_key', 'option');?>&callback=homePageMap&v=weekly"></script>
    </div>
</section>
<?php 
    $review_loc_id = get_field( 'review_loc_id' ); 
    $inclide_recent_posts = get_field( 'include_recent_posts' );
?>
<?php if( $review_loc_id > 0 || $inclide_recent_posts ) { ?>
    <div class="home-featured">
        <div class="aon-container">
            <?php if( $review_loc_id > 0 ) { ?>
            <section>
                <h2 class="section-title">Testimonials</h2>
                <div class="reviews aon-slides">
                <?php
                    $reviews_encoded = file_get_contents("https://www.aoncology.com/reviews.php?loc=".$review_loc_id);
                    $reviews_decoded = json_decode($reviews_encoded,true);

                    $reviews = array();
                    if(!is_null($reviews_decoded)){
                        $reviews=array_slice($reviews_decoded,0,5);
                    }

                    foreach($reviews as $review){
                        ?>
                        <div class="review aon-slide">
                            <div class="date"><?=date('M jS, Y', strtotime($review['datePublished']));?></div>
                            <div class="rating"><?php for($i=0;$i<intval($review['rating']);$i++){echo '★';} ?></div>
                            <div class="description"><?=$review['description'];?></div>
                            <div class="author"><?=$review['author'];?></div>
                        </div>
                        <?php
                    }
                ?>
                </div>
            </section>
            <?php } ?> 
            <?php if( $inclide_recent_posts ) { ?>
            <?php 
                $args = array(
                    'post_type'      => array( 'patient-story', 'news', 'blog' ),
                    'posts_per_page' => 5,
                    'orderby'        => 'date',
                    'order'          => 'DESC',
                );

                $latest_posts = get_posts( $args );
                
            ?>
            <section id="latest-updates">
                <h2 class="section-title">Latest Updates</h2>
                <div class="posts aon-slides">
                    <?php
                        $reviews_encoded = file_get_contents("https://www.aoncology.com/reviews.php?loc=".$review_loc_id);
                        $reviews_decoded = json_decode($reviews_encoded,true);

                        $reviews = array();
                        if(!is_null($reviews_decoded)){
                            $reviews=array_slice($reviews_decoded,0,5);
                        }

                        foreach($latest_posts as $featured_post){
                            $post_type_obj = get_post_type_object( get_post_type( $featured_post ) );
                            $post_type     = $post_type_obj->labels->singular_name;
                            $post_date   = get_the_date( '', $featured_post );
                            $post_title = get_the_title( $featured_post );
                            $post_excerpt = get_the_excerpt( $featured_post );
                            $post_thumbnail = get_the_post_thumbnail_url( $featured_post, 'thumbnail' );                            
                            $post_link   = get_permalink( $featured_post );
                            ?>
                            <div class="post aon-slide">
                                <div class="title"><h3><a href="<?=$post_link;?>"><?=$post_title ?></a></h3></div>
                                <div class="date"><?=date('M jS, Y', strtotime($post_date));?></div>
                                <div class="type"><?= $post_type ?></div>                                
                                <div class="description"><?=$post_excerpt ?></div>
                                <?php if($post_thumbnail){ ?>
                                <img class="thumbnail" src="<?php echo $post_thumbnail; ?>" width="150" height="150" alt="">
                                <?php } ?>
                                <div class="link"><a href="<?=$post_link;?>">Read Article &gt;</a></div>
                            </div>
                            <?php
                        }
                    ?>
                </div>
            </section>
            <?php } ?>
        </div>
    </div>
<?php } ?>
<?php
get_footer();
?>
