<?php

// aon2022theme_setup
function aon2022theme_setup() {

    //Enable support for post thumbnails and featured images.
    add_theme_support( 'post-thumbnails' );


    //custom logo set up		
    add_theme_support( 'custom-logo',
        array(
            'height' => 75,
            'width' => 300,
            'flex-height' => true,
            'flex-width' => true,
            'header-text' => array( 'site-title', 'site-description' ),
            'unlink-homepage-logo' => true,
        )
    );

    //image sizes			
    update_option( 'thumbnail_size_w', 400 );
    update_option( 'thumbnail_size_h', 400 );

    update_option( 'medium_size_w', 800 );
    update_option( 'medium_size_h', 800 );

    update_option( 'large_size_w', 1200 );
    update_option( 'large_size_h', 1200 );

    add_image_size( 'viewport-width', 1900, 1900, false );

    add_filter( 'image_size_names_choose', function ( $sizes ) {
        return array_merge( $sizes, array(
            'viewport-width' => __( 'Viewport Width' ),
        ) );
    } );
    
    remove_filter( 'render_block', 'wp_render_layout_support_flag', 10, 2 );

    //add suport for svg
    add_filter( 'wp_check_filetype_and_ext', function ( $data, $file, $filename, $mimes ) {
        $filetype = wp_check_filetype( $filename, $mimes );
        return [
            'ext' => $filetype[ 'ext' ],
            'type' => $filetype[ 'type' ],
            'proper_filename' => $data[ 'proper_filename' ]
        ];

    }, 10, 4 );

    function cc_mime_types( $mimes ) {
        $mimes[ 'svg' ] = 'image/svg+xml';
        return $mimes;
    }
    add_filter( 'upload_mimes', 'cc_mime_types' );


    //Add support for two custom navigation menus.
    register_nav_menus(
        array(
            'main' => __( 'Main Menu', 'aon2022theme' ),
            'footer_c1' => __( 'Footer Col 1', 'aon2022theme' ),
            'footer_c2' => __( 'Footer Col 2', 'aon2022theme' ),
            'footer_c3' => __( 'Footer Col 3', 'aon2022theme' ),
            'footer_legal' => __( 'Footer Legal', 'aon2022theme' ),
            'quick_links' => __( 'Quick Links Menu', 'aon2022theme' ),
            'social_media' => __( 'Social Media Menu', 'aon2022theme' )
        )
    );
    
    function post_remove ()      //creating functions post_remove for removing menu item
    { 
       remove_menu_page('edit.php');
    }

    add_action('admin_menu', 'post_remove');   //adding action for triggering function cal


    //Dissable emojis		
    remove_action( 'wp_head', 'print_emoji_detection_script', 7 );
    remove_action( 'admin_print_scripts', 'print_emoji_detection_script' );
    remove_action( 'wp_print_styles', 'print_emoji_styles' );
    remove_action( 'admin_print_styles', 'print_emoji_styles' );
    remove_filter( 'the_content_feed', 'wp_staticize_emoji' );
    remove_filter( 'comment_text_rss', 'wp_staticize_emoji' );
    remove_filter( 'wp_mail', 'wp_staticize_emoji_for_email' );

    // Remove from TinyMCE
    function disable_emojis_tinymce( $plugins ) {
        if ( is_array( $plugins ) ) {
            return array_diff( $plugins, array( 'wpemoji' ) );
        } else {
            return array();
        }
    }
    add_filter( 'tiny_mce_plugins', 'disable_emojis_tinymce' );


    // Disable Gutenberg on the back end.
    add_filter( 'use_block_editor_for_post', '__return_false' );

    // Disable Gutenberg for widgets.
    add_filter( 'use_widgets_blog_editor', '__return_false' );

    add_action( 'wp_enqueue_scripts', function () {
        // Remove CSS on the front end.
        wp_dequeue_style( 'wp-block-library' );

        // Remove Gutenberg theme.
        wp_dequeue_style( 'wp-block-library-theme' );

        // Remove inline global CSS on the front end.
        wp_dequeue_style( 'global-styles' );
    }, 20 );

    //Add sitename as a shortcode
    add_shortcode( 'practice_name', function () {
        return get_bloginfo();
    } );
    
    //Add domain as a shortcode
    add_shortcode( 'practice_domain', function () {
        return parse_url( get_site_url(), PHP_URL_HOST );
    } );
    
    //Add first location phone shorcode
    add_shortcode( 'practice_first_location_phone', function () {
        $locations = get_posts( [ 'post_type' => 'location', 'numberposts' => 1 ] );
        if(isset($locations[0])){
            $phone = get_field('phone', $locations[0]);
            return '<a href="tel:' . $phone . '" target="_blank">' . $phone . '</a>';
        }
        return '';
    } );
    
    //clinical trials shortcode
    add_shortcode( 'aon_clinical_trials', function ($atts) {
        if( isset( $atts['facetwpid'] ) ){
            $practice_id = $atts['facetwpid'];
        }else{
            return;
        }
        
        $curl_aon = curl_init();
        curl_setopt($curl_aon, CURLOPT_URL, "https://www.aoncology.com/clinical-trials-practice/?fwp_clinical_trial_practice=" . $practice_id);
        curl_setopt($curl_aon, CURLOPT_RETURNTRANSFER, 1);
        $aon_message = curl_exec($curl_aon);
        curl_close($curl_aon);

        return $aon_message;
    } );
    
    //Add team members shortcode
    add_shortcode( 'physicians', function () {
        $team_members = get_posts( [ 'post_type' => 'physician', 'numberposts' => -1 ] );
        
        ob_start();
        echo '</div><div class="full-width-gap">';
        include('includes/team_members.php');
        echo '</div><div class="aon-container">';
        $html = ob_get_clean();
        
        return $html;
    } );
    
    add_shortcode( 'practitioners', function () {
        $team_members = get_posts( [ 'post_type' => 'practitioner', 'numberposts' => -1 ] );
        
        ob_start();
        echo '</div><div class="full-width-gap">';
        include('includes/team_members.php');
        echo '</div><div class="aon-container">';
        $html = ob_get_clean();
        
        return $html;
    } );
    
    add_post_type_support( 'page', 'excerpt' );
    
    if( function_exists('acf_add_options_page') ) {	
        acf_add_options_page();	
    }
    
    //ACF google maps API Key
    add_filter('acf/fields/google_map/api', function ( $api ){	
        $api['key'] = get_field('google_api_key', 'option');	
        return $api;	
    });
    
    //Include Colors in Code
    get_field('google_api_key', 'option');

}
add_action( 'after_setup_theme', 'aon2022theme_setup' );

function aon_theme_colors_url_handler() {
    if( str_starts_with( $_SERVER["REQUEST_URI"] , '/aon-theme-colors.css' ) ) {
        header('Content-type: text/css');       
        echo '
            :root{
                --primary-compliant-color: ' . get_field('primary_compliant_color', 'option') . ';
                --primary-contrast-color: ' . get_field('primary_contrast_color', 'option') . ';
                --secondary-compliant-color: ' . get_field('secondary_compliant_color', 'option') . ';
                --secondary-contrast-color: ' . get_field('secondary_contrast_color', 'option') . ';
                --tertiary-color: ' . get_field('tertiary_color', 'option') . ';
                --light-gray: ' . get_field('light_gray', 'option') . ';
                --font-color: ' . get_field('font_color', 'option') . ';
                --footer-color: ' . get_field('footer_color', 'option') . ';
                --aon-gap: 1rem;
                --aon-radius: 0.15rem;
            } 
        ';
        exit();
    }
}
add_action('parse_request', 'aon_theme_colors_url_handler');

function aon2022theme_editor_styles() {
    global $editor_styles;
    add_editor_style( 'https://use.typekit.net/jii7eoi.css' );
    add_editor_style( get_site_url().'/aon-theme-colors.css' );
    add_editor_style( 'global-content.css' );
}
add_action( 'init', 'aon2022theme_editor_styles' );

function get_custom_logo_url() {
    $custom_logo_id = get_theme_mod( 'custom_logo' );
    $image = wp_get_attachment_image_src( $custom_logo_id, 'full' );
    return $image[ 0 ];
}


function create_custom_post_types() {

    register_post_type( 'service',
        // CPT Options
        array(
            'public' => true,
            'labels' => array(
                'name' => __( 'Services' ),
                'singular_name' => __( 'Service' ),
                'add_new' => __( 'Add New' ),
                'add_new_item' => __( 'Add New Service' ),
                'edit' => __( 'Edit' ),
                'edit_item' => __( 'Edit Service' ),
                'new_item' => __( 'New Service' ),
                'view' => __( 'View Service' ),
                'view_item' => __( 'View Service' ),
                'search_items' => __( 'Search Services' ),
                'not_found' => __( 'No services found' ),
                'not_found_in_trash' => __( 'No services found in Trash' ),
                'parent' => __( 'Parent Service' ),
            ),
            'supports' => array(
                'title',
                'editor',
                'thumbnail',
                'excerpt',
                'page-attributes'
            ),
        )
    );
    
    register_post_type( 'physician',
        // CPT Options
        array(
            'public' => true,
            'labels' => array(
                'name' => __( 'Physicians' ),
                'singular_name' => __( 'Physician' ),
                'add_new' => __( 'Add New' ),
                'add_new_item' => __( 'Add New Physician' ),
                'edit' => __( 'Edit' ),
                'edit_item' => __( 'Edit Physician' ),
                'new_item' => __( 'New Physician' ),
                'view' => __( 'View Physician' ),
                'view_item' => __( 'View Physician' ),
                'search_items' => __( 'Search Physicians' ),
                'not_found' => __( 'No physicians found' ),
                'not_found_in_trash' => __( 'No physicians found in Trash' )
            ),
            'supports' => array(
                'title',
                'editor',
                'thumbnail',
                'excerpt',
                'page-attributes'
            ),
        )
    );
    
    register_post_type( 'practitioner',
        // CPT Options
        array(
            'public' => true,
            'labels' => array(
                'name' => __( 'Practitioners' ),
                'singular_name' => __( 'Practitioner' ),
                'add_new' => __( 'Add New' ),
                'add_new_item' => __( 'Add New Practitioner' ),
                'edit' => __( 'Edit' ),
                'edit_item' => __( 'Edit Practitioner' ),
                'new_item' => __( 'New Practitioner' ),
                'view' => __( 'View Practitioner' ),
                'view_item' => __( 'View Practitioner' ),
                'search_items' => __( 'Search Practitioners' ),
                'not_found' => __( 'No practitioners found' ),
                'not_found_in_trash' => __( 'No practitioners found in Trash' )
            ),
            'supports' => array(
                'title',
                'editor',
                'thumbnail',
                'excerpt',
                'page-attributes'
            ),
        )
    );
    
    register_post_type( 'location',
        // CPT Options
        array(
            'public' => true,
            'publicly_queryable'  => false,
            'labels' => array(
                'name' => __( 'Locations' ),
                'singular_name' => __( 'Location' ),
                'add_new' => __( 'Add New' ),
                'add_new_item' => __( 'Add New Location' ),
                'edit' => __( 'Edit' ),
                'edit_item' => __( 'Edit Location' ),
                'new_item' => __( 'New Location' ),
                'view' => __( 'View Location' ),
                'view_item' => __( 'View Location' ),
                'search_items' => __( 'Search Locations' ),
                'not_found' => __( 'No locations found' ),
                'not_found_in_trash' => __( 'No locations found in Trash' )
            ),
            'supports' => array(
                'title',
                'excerpt',
                'page-attributes'
            ),
        )
    );
    
    register_post_type( 'news',
        // CPT Options
        array(
            'has_archive' => true,
            'public' => true,
            'labels' => array(
                'name' => __( 'News' ),
                'singular_name' => __( 'News Post' ),
                'add_new' => __( 'Add News Post' ),
                'add_new_item' => __( 'Add News Post' ),
                'edit' => __( 'Edit' ),
                'edit_item' => __( 'Edit News Post' ),
                'new_item' => __( 'New News Post' ),
                'view' => __( 'View News Post' ),
                'view_item' => __( 'View News Post' ),
                'search_items' => __( 'Search News Posts' ),
                'not_found' => __( 'No news posts found' ),
                'not_found_in_trash' => __( 'No news posts found in Trash' ),
                'parent' => __( 'Parent News Post' ),
            ),
            'supports' => array(
                'title',
                'editor',
                'thumbnail',
                'excerpt',
            ),
        )
    );
    
    register_post_type( 'blog',
        // CPT Options
        array(
            'has_archive' => true,
            'public' => true,
            'labels' => array(
                'name' => __( 'Blogs' ),
                'singular_name' => __( 'Blog' ),
                'add_new' => __( 'Add Blog' ),
                'add_new_item' => __( 'Add Blog' ),
                'edit' => __( 'Edit' ),
                'edit_item' => __( 'Edit Blog' ),
                'new_item' => __( 'New Blog' ),
                'view' => __( 'View Blog' ),
                'view_item' => __( 'View Blog' ),
                'search_items' => __( 'Search Blogs' ),
                'not_found' => __( 'No Blogs found' ),
                'not_found_in_trash' => __( 'No Blogs found in Trash' ),
                'parent' => __( 'Parent Blog' ),
            ),
            'supports' => array(
                'title',
                'editor',
                'thumbnail',
                'excerpt',
            ),
        )
    );
    
    register_post_type( 'patient-story',
        // CPT Options
        array(
            'has_archive' => true,
            'public' => true,
            'labels' => array(
                'name' => __( 'Patient Stories' ),
                'singular_name' => __( 'Patient Story' ),
                'add_new' => __( 'Add New' ),
                'add_new_item' => __( 'Add New Patient Story' ),
                'edit' => __( 'Edit' ),
                'edit_item' => __( 'Edit Patient Story' ),
                'new_item' => __( 'New Patient Story' ),
                'view' => __( 'View Patient Story' ),
                'view_item' => __( 'View Patient Story' ),
                'search_items' => __( 'Search Patient Story' ),
                'not_found' => __( 'No patient stories found' ),
                'not_found_in_trash' => __( 'No patient stories found in Trash' )
            ),
            'supports' => array(
                'title',
                'editor',
                'thumbnail',
                'excerpt',
                'page-attributes'
            ),
        )
    );
}
add_action( 'init', 'create_custom_post_types' );

function add_cpt_items_to_menu( $items, $menu, $args ) {
    $new_items = array();
    $menu_order = count( $items );

    foreach ( $items as $item ) {
        foreach ( $item->classes as $class ) {
            if ( substr( $class, 0, 4 ) == 'cpt-' ) {
                $cpt = substr( $class, 4 );
                $parent_item_id = $item->ID;

                foreach ( get_posts( 'post_type=' . $cpt . '&numberposts=-1' ) as $cpt_post ) {
                    $cpt_post->menu_item_parent = $parent_item_id;
                    $cpt_post->post_type = 'nav_menu_item';
                    $cpt_post->object = 'custom';
                    $cpt_post->menu_order = ++$menu_order;                    
                    $cpt_post->type = 'custom';                    
                    $cpt_post->title = $cpt_post->post_title;
                    $cpt_post->url = get_permalink( $cpt_post->ID );
                    
                    //other necesary
                    $cpt_post->object_id = '';
                    $cpt_post->classes = '';
                    $cpt_post->xfn = '';
                    $cpt_post->target = '';
                    $cpt_post->attr_title = '';
                    $cpt_post->description = '';                    
                    
                    array_push( $new_items, $cpt_post );
                }
                continue;
            }
        }
    }

    return array_merge( $items, $new_items );
}

if ( ! is_admin() ) {
    add_filter( 'wp_get_nav_menu_items', 'add_cpt_items_to_menu', 10, 3 );
}

class Main_Menu_Walker extends Walker_Nav_Menu {

    function start_lvl(&$output, $depth = 0, $args = array()) {
        $indent = str_repeat("\t", 5 + $depth + 1 );
        $output .= PHP_EOL . $indent . '<ul role="menu" class="sub-menu hidden" id="sub-menu-' . $args->sub_menu_id . '">';
    }
    
    function start_el(&$output, $item, $depth=0, $args=array(), $id = 0) {
        $indent = str_repeat("\t", 5 + $depth * 2);
        $indent_2 = str_repeat("\t", 5 + $depth * 2 ) . "\t";
        $output .= PHP_EOL . $indent . '<li class="' . implode(' ', $item->classes) . '">';
        
        //Add SPAN if no Permalink
        if( $item->url && $item->url != '#' ) {
            $output .= PHP_EOL . $indent_2 . '<a href="' . $item->url . '" ' . ($item->target == '_blank' ? 'target="_blank" aria-label="' . $item->title . '(opens in new tab)"' : '') . '>' . $item->title . '</a>'; 
        } else if( $args->walker->has_children ) {
            $output .= PHP_EOL . $indent_2 . '<button type="button" aria-haspopup="true" aria-expanded="false" aria-controls="sub-menu-' . $item->ID . '">' . $item->title . '</button>';
        } else {            
            $output .= PHP_EOL . $indent_2 . '<span role="heading">' . $item->title . '</span>';
        }
        
        $args->sub_menu_id = $item->ID;
    }
    
    function end_el(&$output, $item, $depth=0, $args=array(), $id = 0) {
        $indent = str_repeat("\t", 5 + $depth * 2);
        $output .= PHP_EOL . $indent . '</li>';
    }
    
    function end_lvl(&$output, $depth = 0, $args = array()) {
        $indent = str_repeat("\t", 5 + $depth + 1);
        $output .= PHP_EOL . $indent . '</ul>';
    }
}

class Footer_Menu_Walker extends Walker_Nav_Menu {

    function start_lvl(&$output, $depth = 0, $args = array()) {
        $indent = str_repeat("\t", 5 + $depth + 1 );
        $output .= PHP_EOL . $indent . '<ul class="sub-menu">';
    }
    
    function start_el(&$output, $item, $depth=0, $args=array(), $id = 0) {
        $indent = str_repeat("\t", 5 + $depth * 2);
        $indent_2 = str_repeat("\t", 5 + $depth * 2 ) . "\t";
        $output .= PHP_EOL . $indent . '<li class="' . implode(' ', $item->classes) . '">';
        
        //Add SPAN if no Permalink
        if( $item->url && $item->url != '#' ) {
            $output .= PHP_EOL . $indent_2 . '<a href="' . $item->url . '" ' . ($item->target == '_blank' ? 'aria-label="' . $item->title . '(opens in new tab)"' : '') . '>' . $item->title . '</a>'; 
        } else {            
            $output .= PHP_EOL . $indent_2 . '<span>' . $item->title . '</span>';
        }
    }
    
    function end_el(&$output, $item, $depth=0, $args=array(), $id = 0) {
        $indent = str_repeat("\t", 5 + $depth * 2);
        $output .= PHP_EOL . $indent . '</li>';
    }
    
    function end_lvl(&$output, $depth = 0, $args = array()) {
        $indent = str_repeat("\t", 5 + $depth + 1);
        $output .= PHP_EOL . $indent . '</ul>';
    }
}
