window.addEventListener('load', function () {
    /*document.body.classList.remove("prevent-transitions");*/

    /** Header onscreen observer */
    const headerObserver = document.getElementById("header-observer");
    const header = document.querySelector("header");

    function obCallback(payload) {
        //console.log(payload[0].isIntersecting);
        if (payload[0].isIntersecting) {
            header.classList.remove("with-background");
        } else {
            header.classList.add("with-background");
        }
    }

    const ob = new IntersectionObserver(obCallback);

    ob.observe(headerObserver);

    document.querySelectorAll('#menu-main a').forEach((anchor) =>{
        anchor.addEventListener('click',(e) =>{
            if(anchor.href.slice(-2)=='/#'){
                e.preventDefault;
                return true;
            }
            document.querySelector('#menu-main-expand').checked=false;                        
        });    
    });


    /* Menu Toogle */
    let toggleButtons = document.querySelectorAll( '[aria-haspopup]' );
    toggleButtons.forEach( toggle => {
        const controlledElement = document.getElementById( toggle.getAttribute( 'aria-controls' ) );
        toggle.addEventListener( 'click', () => {
            if( toggle.getAttribute( 'aria-expanded' ) === "true" ) {
                toggle.setAttribute( 'aria-expanded', "false" );
                controlledElement.classList.add( 'hidden' );
            } else {
                toggle.setAttribute( 'aria-expanded', "true" );
                controlledElement.classList.remove( 'hidden' );
            }
        } );
    } );

    /* Menu hover on desktop */
    let navListItems = document.querySelectorAll( '#menu-main > li' );
    navListItems.forEach( listItem => {
        listItem.addEventListener( "mouseover", () => {
            if( window.matchMedia( "(max-width: 55rem)" ).matches) {
                return;
            }
            const button = listItem.querySelector( ':scope > button' );
            if( button.getAttribute('aria-expanded') === 'false'){
                button.click();
            }
        } );
        listItem.addEventListener( "mouseout", () => {
            if( window.matchMedia( "(max-width: 55rem)" ).matches ) {
                return;
            }
            const button = listItem.querySelector( ':scope > button' );
            if( button.getAttribute('aria-expanded') === 'true'){
                button.click();
            }
        } );
    } );

    /** slider by hand */
    document.querySelectorAll('.slider').forEach( (slider) => {
        let mouseDown = false;
        let startX, scrollLeft;

        slider.style.cursor = 'grab';
        //slider.style.overflow = 'hidden';
        //slider.style['scroll-snap-type'] = 'none';
        //slider.style['scroll-behavior'] = 'auto';


        let startDragging = function (e) {
            mouseDown = true;
            startX = e.pageX - slider.offsetLeft;
            scrollLeft = slider.scrollLeft;

            slider.style.cursor = 'grabbing';
            slider.style['scroll-snap-type'] = 'none';
            slider.style['scroll-behavior'] = 'auto';
        };

        let stopDragging = function (e) {
            mouseDown = false;

            slider.style.cursor = 'grab';
            //slider.style['scroll-snap-type'] = '';
            slider.style['scroll-behavior'] = '';
        };

        let clickWhileDraggring = function (e) {
            const x = e.pageX - slider.offsetLeft;
            if(x != startX){
                e.preventDefault();
            }
        }

        slider.addEventListener('mousemove', (e) => {
            e.preventDefault();
            if (!mouseDown) {
                return;
            }
            const x = e.pageX - slider.offsetLeft;
            const scroll = x - startX;
            slider.scrollLeft = scrollLeft - scroll;
        });

        // Add the event listeners
        slider.addEventListener('mousedown', startDragging, false);
        slider.addEventListener('click', clickWhileDraggring, false);
        slider.addEventListener('mouseup', stopDragging, false);
        slider.addEventListener('mouseleave', stopDragging, false);
    } );
    
    /** Scroll to section **/
    const hash = window.location.hash;
    if (hash !== "") {
        window.requestAnimationFrame(()=>{
            const targetElement = document.querySelector(hash);
            if (targetElement) {
                const headerHeight = document.querySelector('header').offsetHeight;
                const offsetTop = targetElement.offsetTop - headerHeight;
                window.scrollTo({
                    top: offsetTop,
                    behavior: 'smooth'
                });
            }
        });
    }
});